# Estimate two different models on the Lorartan data

# load libraries and functions
library(splines)
library(jagsUI)
source('customspline.R')

DETECTION_LIMIT <- 2
N_DRUG <- 2

# load data
Data <- read.table("LosartanData.txt", header=TRUE)
time <- unique(Data$Time1)
Ids  <- unique(Data$Vol)
max.time <- length(time)
max.id <- length(Ids)
Y1 <- matrix(Data$Conc1, max.id, max.time, byrow = TRUE)
Y2 <- matrix(Data$Conc2, max.id, max.time, byrow = TRUE)
# we place all observations in a matrix elements are y[i, j, k]
# with i individual, j observation and k drug
Y <- array(c(Y1, Y2), c(dim(Y1), N_DRUG))
Yo <- ifelse(Y>=DETECTION_LIMIT, Y, NA)    # observed outcomes
# define some starting values for censored observations
Yc <- ifelse(Y < DETECTION_LIMIT,
             ifelse(rep(1:max.time, each=max.id) %in% c(1, max.time),
                    0,
                    DETECTION_LIMIT/4),
             NA) 
# define vector of order that can be indexed by the id of the volunteer
med.order <- Data$Order[!duplicated(Data$Vol)]

# Define spline 
knots<-c(0.4, 1.2, 3.5, 5, 8, 14)
num.knots<-length(knots)
grid<-c(0, 0.0025, seq(from=0.005, to=0.995, 0.005),seq(from=1,to=3.96, 0.04), seq(4,6.9, 0.1), seq(7, 24, 0.2))
grid2<- c(-1000, grid, 1000) # expand the grid with two very remote klots to prevent extrapolation problems
boundary.knots<-c(0, 24)
B <- ns(grid,knots = knots, Boundary.knots=boundary.knots, intercept = TRUE)
B2<-custom.spline(grid2,knots = knots, Boundary.knots=boundary.knots, intercept = TRUE,
                  left.extra=0,
                  restrictions=matrix(c(0, 0, 24, 24, 24, 2, 0, 2, 1, 0), ncol = 2) )


# Setup parametric model
Omega <-diag(rep(0.01,6)) ## Omega will be the inverse of the variance matrix!

cat("
 model{
  for(i in 1:nsub)    {
      b[i,1:6] ~ dmnorm(mu.b[1:6], tau.b[1:6, 1:6])
      lKe[1, i] <- min(phi1[1, i], phi2[1, i])
      lKa[1, i] <- max(phi1[1, i], phi2[1, i])
      lKe[2, i] <- min(phi1[2, i], phi2[2, i])
      lKa[2, i] <- max(phi1[2, i], phi2[2, i])
      Ke[1, i] <- exp(lKe[1, i])
      Ka[1, i] <- exp(lKa[1, i])
      Ke[2, i] <- exp(lKe[2, i])
      Ka[2, i] <- exp(lKa[2, i])
      phi1[1, i] <- beta1[1] + b[i, 1] * xi[1]
      phi2[1, i] <- beta1[2] + b[i, 2] * xi[2]
      phi3[1, i] <- beta1[3] + b[i, 3] * xi[3]
      phi1[2, i] <- beta2[1] + b[i, 4] * xi[4]
      phi2[2, i] <- beta2[2] + b[i, 5] * xi[5]
      phi3[2, i] <- beta2[3] + b[i, 6] * xi[6]
  }  # for(i in 1:nsub) 
  tau.b[1:6, 1:6] ~ dwish(6 * Omega[ , ], 6)
  for(j in 1:n.time){
    for(i in 1:nsub){
      for(k in 1:2){
        observed[i, j, k] ~ dinterval(y[i, j, k], 2)
        y[i, j, k] ~ dnorm(mu[i, j, k], tau[k]) #data model: the likelihood
        mu[i, j, k] <- exp(lKe[k, i]+lKa[k, i]-phi3[k, i] +
        alpha[k] *order[i]) *
        (exp(-Ke[k, i]*time[j])-exp(-Ka[k, i]*time[j])) / 
        (Ka[k, i]-Ke[k, i])
        res[i, j, k] <- y[i, j, k] - mu[i, j, k] 
      } #  for(k in 1:2)
    } #  for(j in 1:nsub)
  }  # for(i in 1:n.time){
  for(k in 1:2){
    tau[k] ~ dgamma(0.001, 0.001)
    alpha[k] ~ dnorm(0,0.0001)
  }  
  for(i in 1:3){
    beta1[i] ~ dnorm(0, 0.0001)
    beta2[i] ~ dnorm(0, 0.0001)
  }
  for (i in 1:6){
    xi[i] ~ dunif(0, 100)
  }
  }"
,file="mcomp.bug")


n.thin <- 300
n.iter <- 8000 * n.thin
n.burnin <-2000 * n.thin


jags.params <- c( "beta1", "beta2", "tau", "tau.b","mu", "res", "b",'shift1','shift2', 'xi',
                  'alpha', 'Ke', 'Ka', 'phi', 'lKe', 'lKa')

my.data <- list("y"=Yo,'observed' =ifelse(is.na(Yo), 0, 1),
                "time"=time,
                "nsub"=max.id, "n.time"=max.time,   
                "Omega"=Omega,"mu.b"=rep(0, 6),
                order=med.order)

jags.inits <- function(){list("beta1"=c(0.5, 0.2, -5.7), "beta2"=c(0.5, 0.2, -5.7),
                              "tau"=c(0.01, 0.01), 'alpha'=c(0.15, 0.15),
                              'xi'=rep(1, 6), "y"=Yc)}


jagsfitComp <- jagsUI::jags(data=my.data, inits=jags.inits, parameters.to.save=jags.params, model.file="mcomp.bug",
                         n.chains=3, DIC=TRUE, n.adapt=1000, n.iter = n.iter, n.burnin=n.burnin, n.thin=n.thin,
                         seed = 123,  parallel=TRUE)   

save(jagsfitComp ,file='jagsfitComp.Rdata')
# Estimate MSITAR model
cat("
    model{
    #subject specific effects
    tau.b[1:6, 1:6] ~ dwish(6 * Omega[,], 6)
    for (m in 1:6){
      xi[m] ~ dunif(0, 100)
      gammas[i, m] <- b[i, m] * xi[m] 
    }
    for(i in 1:nsub) {
      b[i, 1:6] ~ dmnorm(mu.b[1:6], tau.b[1:6, 1:6])
      shift1[i] <- min(gammas[i, 5], 0)
      shift2[i] <- min(gammas[i, 6], 0)
      for(j in 1:n.time){           
        x[i,j,1] <- exp(gammas[i,1]) * (time[j] + shift1[i])
        x[i,j,2] <- exp(gammas[i,2]) * (time[j] + shift2[i])   
        mu[i, j, 1] <- exp(gammas[i, 3] + alpha[1] * order[i]) *
        inprod(beta1[1:L], B[i,j, 1:L,1])   
        mu[i, j, 2] <- exp(gammas[i, 4] + alpha[2] * order[i])*
        inprod(beta2[1:L], B[i,j, 1:L,2])
        for(k in 1:2){
          y[i, j, k] ~ dnorm(mu[i, j, k], tau[k] )  
          res[ i , j, k] <- y[i, j, k] - mu[i, j, k]
          observed[i, j, k] ~ dinterval(y[i, j, k], 2)
          for(l in 1:L){
           B[i, j, l, k] <- interp.lin(x[i, j, k], grid, Bas[ , l])
          } # for l    
        } # for k
      } # for j
    } # for i              
    for(k in 1:2){
      tau[k] ~ dgamma(0.001, 0.001)
      alpha[k] ~ dnorm(0,0.0001)
    }  
    for(l in 1:L){
      beta1[l] ~ dnorm(0, 0.0001)     
      beta2[l] ~ dnorm(0, 0.0001)  
    }    
  }"
,file="mSITAR.bug")

my.data <- list("y"=Yo, 'observed'=ifelse(Y<2, 0, 1),
                "time"=time,
                "nsub"=max.id, "n.time"=max.time,   
                "L"=NCOL(B2), "Omega"=Omega, "mu.b"=rep(0, 6),
                "grid"=grid2, "Bas"=B2,
                order=Data$Order[seq(1, max.time *max.id, max.time)])

jags.params <- c( "beta1", "beta2", "tau", "tau.b","mu", "res", "b",'shift1','shift2', 'xi',
                  'alpha')


jags.inits <- function(){list("beta1"=c(14.5, 14, -54, 127, -147),
                              "beta2"=c(13, 14, -50, 128, -150),
                              "tau"=c(0.01, 0.01),
                              "b"=matrix(0,24,6), 'xi'=rep(1, 6),
                              "y"=Yc)}


jagsfitSITAR <- jagsUI::jags(data=my.data, inits=jags.inits, parameters.to.save=jags.params, model.file="mSITAR.bug",
                             n.chains=3, DIC=TRUE, n.adapt=8000, n.iter = n.iter, n.burnin=n.burnin, n.thin=n.thin , seed = 123,  parallel=TRUE)   
save(jagsfitSITAR,file='jagsfitSITAR.Rdata')
